<?php

require_once("Router.php");
require_once("model/Color.php");

class MainView {

	protected $router;
	protected $style;
	protected $title;
	protected $content;

	public function __construct(Router $router) {
		$this->router = $router;
		$this->style = "";
		$this->title = null;
		$this->content = null;
	}


	/******************************************************************************/
	/* Méthodes de génération des pages                                           */
	/******************************************************************************/

	public function makeHomePage() {
		$this->title = "Proposez vos couleurs !";
		$this->content = "Bienvenue sur ce site de partage de couleurs.";
	}

	public function makeColorPage($id, Color $c) {
		$cname = self::htmlesc($c->getName());
		$chex = $c->getHex();
		$crgb = $c->getRGB();
		$chsl = $c->getHSL();
		$cclass = "color$id";
		$cdatec = self::fmtDate($c->getCreationDate());
		$cdatem = self::fmtDate($c->getModifDate());

		$this->style .= ".$cclass { background-color: #$chex; }";
		$this->title = "La couleur $cname";
		$s = "";
		$s .= "<figure>\n<div class=\"sample $cclass\"></div>\n";
		$s .= "<figcaption>Un échantillon de $cname.</figcaption>\n</figure>\n";
		$s .= "<p>La couleur $cname a pour code hexadécimal $chex.</p>\n";
		$s .= "<p>Elle contient du rouge à ".round($crgb[0]*100/255)."%, du vert à ";
		$s .= round($crgb[1]*100/255)."%, et du bleu à ".round($crgb[2]*100/255)."%.</p>";
		$s .= "<p>Sa teinte est de ".$chsl[0]."°, avec une saturation de ".$chsl[1];
		$s .= "% et une luminosité de ".$chsl[2]."%.</p>";
		$s .= "<p>Elle a été créée ".$cdatec." et modifiée ".$cdatem."</p>\n";
		$this->content = $s;
	}

	public function makeGalleryPage(array $colors) {
		$this->title = "Toutes les couleurs";
		$this->content = "<p>Cliquer sur une couleur pour voir des détails.</p>\n";
		$this->content .= "<ul class=\"gallery\">\n";
		foreach ($colors as $id=>$c) {
			$this->content .= $this->galleryColor($id, $c);
		}
		$this->content .= "</ul>\n";
	}

	public function makeUnknownColorPage() {
		$this->title = "Erreur";
		$this->content = "La couleur demandée n'existe pas.";
	}

	public function makeUnknownActionPage() {
		$this->title = "Erreur";
		$this->content = "La page demandée n'existe pas.";
	}

	/* Génère une page d'erreur inattendue. Peut optionnellement
	 * prendre l'exception qui a provoqué l'erreur
	 * en paramètre, mais n'en fait rien pour l'instant. */
	public function makeUnexpectedErrorPage(Exception $e=null) {
		$this->title = "Erreur";
		$this->content = "Une erreur inattendue s'est produite.";
	}

	/******************************************************************************/
	/* Méthodes utilitaires                                                       */
	/******************************************************************************/

	protected function getMenu() {
		return array(
			"Accueil" => $this->router->homePage(),
			"Couleurs" => $this->router->allColorsPage(),
		);
	}

	protected function galleryColor($id, $c) {
		$cclass = "color".$id;
		$this->style .= '.'.$cclass.' { background-color: #'.$c->getHex().'; }';
		$res = '<li><a href="'.$this->router->colorPage($id).'">';
		$res .= '<h3>'.self::htmlesc($c->getName()).'</h3>';
		$res .= '<div class="sample '.$cclass.'"></div>';
		$res .= '</a></li>'."\n";
		return $res;
	}

	protected static function fmtDate(DateTime $date) {
		return "le " . $date->format("Y-m-d") . " à " . $date->format("H:i:s");
	}

	/* Une fonction pour échapper les caractères spéciaux de HTML,
	* car celle de PHP nécessite trop d'options. */
	public static function htmlesc($str) {
		return htmlspecialchars($str,
			/* on échappe guillemets _et_ apostrophes : */
			ENT_QUOTES
			/* les séquences UTF-8 invalides sont
			* remplacées par le caractère �
			* au lieu de renvoyer la chaîne vide…) */
			| ENT_SUBSTITUTE
			/* on utilise les entités HTML5 (en particulier &apos;) */
			| ENT_HTML5,
			'UTF-8');
	}

	/******************************************************************************/
	/* Rendu de la page                                                           */
	/******************************************************************************/

	public function render() {
		if ($this->title === null || $this->content === null) {
			$this->makeUnexpectedErrorPage();
		}
		/* On affiche la page.
		 * Ici on pourrait faire des echo, mais simplement fermer
		 * la balise PHP revient au même, et le HTML est plus lisible.
		 * En revanche le code PHP est moins lisible : une autre solution
		 * est de mettre ce squelette dans un fichier à part et
		 * de simplement faire un «include» (c'est ce qui a été fait pour 
		 * le site des poèmes). */
?>
<!DOCTYPE html>
<html lang="fr">
<head>
	<title><?php echo $this->title; ?></title>
	<meta charset="UTF-8" />
	<link rel="stylesheet" href="skin/screen.css" />
	<style>
<?php echo $this->style; ?>
	</style>
</head>
<body>
	<nav class="menu">
		<ul>
<?php
/* Construit le menu à partir d'un tableau associatif texte=>lien. */
foreach ($this->getMenu() as $text => $link) {
	echo "<li><a href=\"$link\">$text</a></li>";
}
?>
		</ul>
	</nav>
	<main>
		<h1><?php echo $this->title; ?></h1>
<?php
echo $this->content;
?>
	</main>
</body>
</html>
<?php /* fin de l'affichage de la page et fin de la méthode render() */

	}

}

?>
